define([
	"dojo/_base/declare",
	"dojo/Deferred",
	"dojo/Stateful",
	"dojox/mvc/at",
	"aps/ResourceStore",
	"aps/Tile",
	"aps/Output",
	"aps/common",
	"aps/nav/ViewPlugin"
], function (declare, Deferred, Stateful, at, Store, Tile, Output, common, ViewPlugin) {
   return declare(ViewPlugin, {
      init: function (mediator) {

		/* Initialize variables and the widget */
		var self = this,
			vpsCount = new Stateful({ usage: 0, limit: 0 }),
			widget = new Tile({
				id: "vpsList",
				title: "VPSes",
				onClick: function () {
					aps.apsc.gotoView("http://aps-standard.org/samples/basic1pdash#servers");
				},
				buttons: [{
					title: "Create new VPS",
					iconClass: "fa-plus",
					autoBusy: false,
					type: "link",
					onClick: function () {
						aps.apsc.gotoView("http://aps-standard.org/samples/basic1pdash#vps-wizard");
					}
				}]
			});

		/* Define the function that returns the Tile widget */
		mediator.getWidget = function () {
			return widget;
		};

		/* Arrange continuous monitoring of subscriptions.
		Update the widget respectively. */
		mediator.watch("resourceUsage", function (name, oldvalue, newvalue) {
			if(newvalue !== undefined) {
				common.forEach(newvalue, function(subscriptionResources) { // All resources
					//of a subscription
					subscriptionResources.forEach(function(resource) {
						if(resource.apsType === "http://aps-standard.org/samples/basic1pdash/vps/1.0") {
							vpsCount.set({
								"usage": resource.usage,
								"limit": resource.limit
							});
						}
					});
				});
			}
			else {
				vpsCount.set({
					"usage": 0,
					"limit": 0
				});
			}
			var output = self.byId("dash_vps_plugin_out");
	
				if(!output) {
					output = new Output({
						id: "dash_vps_plugin_out",
						value: at(vpsCount, "usage"),
						limit: at(vpsCount, "limit"),
						content: "Total: ${value} servers out of ${limit}"
					});
					widget.addChild(output);
				}
			   else {
			      output.set({
			         content: "Total: ${value} servers out of ${limit}"
			      });
			   }
		});

      }
   });
});
