define([
  "dojo/_base/declare",
  "dojo/when",
  "aps/View",
  "aps/ResourceStore",
  "./displayError"
], function (declare, when, View, Store, displayError) {
	var self;   // Global variable to exchange data between methods
	return declare(View, {
		init: function() {
			self = this;   // Refer to the declared View object
			/* Define the data store */
			var store = new Store({
				apsType: "http://aps-standard.org/samples/basic1pdash/vps/1.0",
				target: "/aps/2/resources/"
			});
			
			/* Define widget processing logic  */
			var add = function() {
				/* Start the process of creating a VPS by going to the first screen */
				aps.apsc.gotoView("vps-wizard");
			};
			var refresh = function() {
			    self.grid.refresh();
			};
			function changeState(state, btn) {
			    /* Get the list of selected VPSes */
			    var sel = self.grid.get("selectionArray"),
			    counter = sel.length;

			    /* Clear the current messages on the screen */
			    self.page.get("messageList").removeAll(); // The page widget is defined by the ``onContext`` method

			    sel.forEach(function(vpsId){
			       console.log("I'm trying to change state of VPS with id = [" + vpsId + "]");

			       var vps = {
			          aps: { id: vpsId },
			          state: state
			       };

			       /* Save the VPS properties */
			       when(store.put(vps),
			           /* If success, process the next VPS until the list is empty */
			          function() {
			             console.log("State of VPS with id = [" + vpsId + "] changed");
			             /* Remove the processed VPS from the selection */
			             sel.splice(sel.indexOf(vpsId),1);
			             self.grid.refresh();
			             /* Remove busy state of the button */
			             if (--counter === 0) { btn.cancel(); }
			          },
			          /* If failure, call the error handler */
			          function(e) {
			             displayError(e);
			             if (--counter === 0) { btn.cancel(); }
			          }
			       );
			    });
			}
			var stop = function() {
			      changeState("Stopped", this);
			};
			var start = function() {
			      changeState("Running", this);
			};
			var remove = function() {
			    var btn = this;
			    /* Get confirmation from the user for the delete operation */
			    if (!confirm("Are you sure you want delete VPSes?")) {
			       btn.cancel();
			       return;
			    }

			    var sel = self.grid.get("selectionArray");
			    var counter = sel.length;

			    /* Clear the current messages on the screen */
			    self.page.get("messageList").removeAll();

			    sel.forEach(function(vpsId){
			       console.log("I'm trying to delete VPS with id = [" + vpsId + "]");

			       /* Remove the VPS from the APS controller DB */
			       when(store.remove(vpsId),
			           /* If success, process the next VPS until the list is empty */
			          function(){
			             console.log("VPS with id = [" + vpsId + "] removed");
			             /* Remove the processed VPS from the selection */
			             sel.splice(sel.indexOf(vpsId),1);
			             self.grid.refresh();
			             if (--counter === 0) { btn.cancel(); }
			          },
			          /* If failure, call the error handler */
			          function(e){
			             displayError(e);
			             if (--counter === 0) { btn.cancel(); }
			          }
			       );
			    });
			};
			
			/* Define and return widgets */
			return ["aps/Grid", {
			      id:                this.genId("srv_grid"),
			      store:             store,
			      selectionMode:     "multiple",
			      apsResourceViewId: "server-edit",
			      noDataText: "No servers provisioned",
			      noEntriesFoundText: "No servers meet your search criteria",
			      columns:           [{
			                     field: "name",
			                     name: "Name",
			                     type: "resourceName",
			                     filter: true
			             }, {
			                     field: "hardware.memory",
			                     name: "RAM",
			                     title: "RAM",
			                     type: "integer",
			                     filter: true
			             }, {
			                     field: "hardware.diskspace",
			                     name: "Disk Space",
			                     title: "Disk",
			                     type: "integer",
			                     filter: true
			             }, {
			                     field: "hardware.CPU.number",
			                     name: "CPU",
			                     type: "integer",
			                     filter: true
			             }, {
			                     field: "platform.OS.name",
			                     name: "OS",
			                     filter: true
			             }, {
			                     field: "state",
			                     name: "State",
			                     filter: true
			             }]}, [

			         ["aps/Toolbar", [
			               ["aps/ToolbarButton", {
			                    id: this.genId("srv_new"),
			                    iconClass:"fa-plus",
			                    type: "primary",
			                    label: "New",
			                    onClick: add
			               }],
			               ["aps/ToolbarButton", {
			                    id: this.genId("srv_start"),
			                    iconClass:"fa-play",
			                    type: "success",
			                    label: "Start",
			                    requireItems: true,
			                    onClick: start
			               }],
			               ["aps/ToolbarButton", {
			                    id: this.genId("srv_stop"),
			                    iconClass:"fa-pause",
			                    type: "warning",
			                    label: "Stop",
			                    requireItems: true,
			                    onClick: stop
			               }],
			               ["aps/ToolbarButton", {
			                    id: "srv_delete",
			                    iconClass:"fa-trash",
			                    type: "danger",
			                    label: "Delete",
			                    requireItems: true,
			                    onClick: remove
			               }],
			               ["aps/ToolbarButton", {
			                    id: this.genId("srv_refresh"),
			                    iconClass:"fa-refresh",
			                    label: "Refresh",
			                    autoBusy: false,
			                    onClick: refresh
			               }]
			         ]]
			]];
		},

		onContext: function() {
			self.grid = this.byId("srv_grid");
			self.page = this.byId("apsPageContainer");
			self.grid.refresh();
		    aps.apsc.hideLoading();
		},
		
		onHide: function() {
			this.byId("srv_new").cancel();
		}
});
});
